unit FancyLedButton;

{-----------------------------------------------
(* TFancyLedButton 1.0 - button with optional:                    *)
(* - sape (oval, Rectangle, Ellipse and Capsule);                 *)
(* - gradient effect surrounding the border;                      *)
(* - rised, lowered or shadowed multiline caption;                *)
(* - customizable led.                                            *)
(*    Based on DsFancyButton by Djoko Susilo and RxLib sources.   *)
(*  (c) 2001. Alek Terekhov. e-mail: alekn@mail.ru                *) 

{$R-}
{$B-}
{$H+}   {--always use this compiler directive----}

{--for Delphi 5 only--}
{$DEFINE Delphi5} {--disable this line for previous Delphi version--}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, HintProp, DsgnIntf;

type
  {--Fancy options--}
  TTextStyle = (txNone, txLowered, txRaised, txShadowed);
  TBtnShape = (btnCapsule, btnOval, btnRectangle);
  TFrameStyle = (fmNone, fmGradient);
  TLedEffect = class(TPersistent)
  private
    FButtonColor: TColor;
    FBtnShape: TBtnShape;
    FCornerRadius: Integer;
    FFrameColor: TColor;
    FFrameWidth: Integer;
    FHoverColor: TColor;
    FShadowColor: TColor;
    FTextStyle: TTextStyle;
    FFrameStyle: TFrameStyle;
    FOnEffectChange: TNotifyEvent;
    procedure DoChanges;
  protected
    procedure SetButtonColor(Value: TColor);
    procedure SetBtnShape(Value: TBtnShape);
    procedure SetCornerRadius(Value: Integer);
    procedure SetFrameColor(Value: TColor);
    procedure SetShadowColor(Value: TColor);
    procedure SetFrameStyle(Value: TFrameStyle);
    procedure SetFrameWidth(Value: Integer);
    procedure SetHoverColor(Value: TColor);
    procedure SetTextStyle(Value: TTextStyle);
  public
  published
    property FaceColor: TColor read FButtonColor write SetButtonColor;
    property FrameColor: TColor read FFrameColor write SetFrameColor;
    property FrameStyle: TFrameStyle read FFrameStyle write SetFrameStyle;
    property FrameWidth: Integer read FFrameWidth write SetFrameWidth;
    property ShadowColor: TColor read FShadowColor write SetShadowColor;
    property HoverColor: TColor read FHoverColor write SetHoverColor;
    property Shape: TBtnShape read FBtnShape write SetBtnShape;
    property CornerRadius: Integer read FCornerRadius write SetCornerRadius;
    property TextStyle: TTextStyle read FTextStyle write SetTextStyle;
    property OnEffectChange: TNotifyEvent read FOnEffectChange write FOnEffectChange;
  end;

  {--LED options--}
  TDsLayout = (lyLeft, lyTop, lyRight, lyBottom);
  TDsLed = class(TPersistent)
  private
    FLedColor : TColor;
    FLayot : TDsLayout;
    FLedHighlightColor : TColor;
    FLedBorderColor : TColor;
    FLedBorderSize : integer;
    FLedSize : integer;
    FOnLedChange : TNotifyEvent;
    FLedEnabled : boolean;
    FLedSpace: Integer;

  protected
    procedure SetLedColor (Value : TColor);
    procedure SetLedBorderColor (Value : TColor);
    procedure SetLedBorderSize (Value : integer);
    procedure SetLedSize (Value : integer);
    procedure SetLedLayout (Value : TDsLayout);
    procedure SetLedSpace(Value: Integer);
    procedure SetLedEnabled (Value : boolean);

    procedure DoChanges;

  published
    property LedSize : integer read FLedSize write SetLedSize;
    property LedColor : TColor read FLedColor  write SetLedColor;
    property LedHighlightColor : TColor read FLedHighlightColor write FLedHighlightColor;
    property LedBorderColor : TColor read FLedBorderColor write SetLedBorderColor;
    property LedBorderSize : integer read FLedBorderSize write SetLedBorderSize;
    property Enabled : boolean read FLedEnabled write SetLedEnabled;
    property OnLedChange: TNotifyEvent read FOnLedChange write FOnLedChange;
    property Distance: Integer read FLedSpace write SetLedSpace;
    property Layout: TDsLayout read FLayot write SetLedLayout;
  end;

  {--Fancy button--}
  TFancyLedButton = class(TGraphicControl)
  private
    FEffect: TLedEffect;
    FAutoSize : boolean;
    FLed : TDsLed;
    FIsDown: Boolean;
    FInRegion: Boolean;
    FLedL, FLedT: Integer;

    FTextL, FTextT, FTextR: Integer;

    procedure CMMouseLeave(var AMsg: TMessage); message CM_MOUSELEAVE;
    procedure CMMouseEnter(var AMsg: TMessage); message CM_MOUSEENTER;
    procedure CMEnabledChanged(var message: TMessage);
              message CM_ENABLEDCHANGED;
    procedure CMTextChanged(var message: TMessage);
              message CM_TEXTCHANGED;
    procedure CMDialogChar(var message: TCMDialogChar);
              message CM_DIALOGCHAR;
    procedure UpdateChanges(Sender: TObject);
    procedure SetAutoSize (Value : boolean);
    function SpotRGB(Color1, Color2 : TColor; Level: Byte): TColor;
  protected
    procedure Click; override;
    procedure DrawShape;
    procedure DrawShapePressed;
    procedure DrawButtonArea;
    procedure DrawLed;
    procedure WriteCaption;
    procedure LayoutSetting;
    procedure Paint; override;
    function IsInRegion(X, Y: Integer): Boolean;

    procedure MouseUp(Button: TMouseButton;
              Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton;
              Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    {$IFDEF Delphi5}
    property Anchors;
    property Constraints;
    {$ENDIF}

    property Caption;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FancyEffects: TLedEffect read FEffect write FEffect;
    property Led: TDsLed read FLed write FLed;
    property Font;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property AutoSize : boolean read FAutoSize write SetAutoSize;

    property OnClick;         property OnDragDrop;
    property OnDragOver;      property OnEndDrag;
    property OnMouseDown;     property OnMouseUp;
    property OnMouseMove;
  end;

procedure Register;

implementation

{----------------------------}
{--Fancy Options-------------}
{----------------------------}

procedure TLedEffect.SetButtonColor(Value: TColor);
begin
  if value<>FButtonColor then
    begin FButtonColor:=Value; DoChanges; end;
end;

procedure TLedEffect.SetFrameColor(Value: TColor);
begin
  if Value<>FFrameColor then
    begin FFrameColor:=Value; DoChanges; end;
end;

procedure TLedEffect.SetHoverColor(Value: TColor);
begin
  if Value<>FHoverColor then
    begin FHoverColor:=Value; DoChanges; end;
end;

procedure TLedEffect.SetShadowColor(Value: TColor);
begin
  if Value<>FShadowColor then
    begin FShadowColor:=Value; DoChanges; end;
end;

procedure TLedEffect.SetBtnShape(Value: TBtnShape);
begin
  if Value<>FBtnShape then
    begin FBtnShape:=Value; DoChanges; end;
end;

procedure TLedEffect.SetCornerRadius(Value: Integer);
begin
  if Value<>FCornerRadius then
  begin
    FCornerRadius:=Value;
    if FCornerRadius<0 then FCornerRadius:=0;
    if FCornerRadius>145 then FCornerRadius:=145;
    DoChanges;
  end;
end;

procedure TLedEffect.SetFrameWidth(Value: Integer);
begin
  if Value<>FFrameWidth then
  begin
    FFrameWidth:=Value;
    if FFrameWidth<3 then FFrameWidth:=3;
    if FFrameWidth>14 then FFrameWidth:=14;
    DoChanges;
  end;
end;

procedure TLedEffect.SetTextStyle(Value: TTextStyle);
begin
  if Value<>FTextStyle then
    begin FTextStyle:=Value; DoChanges; end;
end;

procedure TLedEffect.SetFrameStyle(Value: TFrameStyle);
begin
  if Value<>FFrameStyle then
    begin FFrameStyle:=Value; DoChanges; end;
end;

procedure TLedEffect.DoChanges;
begin
  if Assigned(FOnEffectChange) then FOnEffectChange(Self);
end;


{----------------------------}
{--Led Options-------------}
{----------------------------}
procedure TDsLed.SetLedColor(Value: TColor);
begin
    if (Value <> FLedColor) then
    begin
        FLedColor := Value;
        DoChanges;
     end;
end;

procedure TDsLed.SetLedLayout(Value: TDsLayout);
begin
  if Value<>FLayot then
  begin
    FLayot:=Value;
    DoChanges;
  end;
end;

procedure TDsLed.SetLedEnabled(Value: boolean);
begin
  if Value<>FLedEnabled then
  begin
    FLedEnabled:=Value;
    DoChanges;
  end;
end;

procedure TDsLed.SetLedSpace(Value: Integer);
begin
  if Value<>FLedSpace then
  begin
    FLedSpace:=Value;
    if FLedSpace<1 then FLedSpace:=0;
    DoChanges;
  end;
end;

procedure TDsLed.SetLedBorderColor(Value : TColor);
begin
  if Value<>FLedBorderColor then
  begin
    FLedBorderColor:=Value;
    DoChanges;
  end;
end;

procedure TDsLed.SetLedBorderSize(Value: integer);
begin
  if Value<>FLedBorderSize then
  begin
    if Value < fLedSize div 2 then
       FLedBorderSize := Value
          else FLedBorderSize := FLedSize div 2 - 3;
    DoChanges;
  end;
end;

procedure TDsLed.SetLedSize(Value: Integer);
begin
  if Value<>FLedSize then
  begin
    FLedSize:=Value;
    DoChanges;
  end;
end;

procedure TDsLed.DoChanges;
begin
  if Assigned(FOnLedChange) then FOnLedChange(Self);
end;


{----------------------------}
{--Fancy Button--------------}
{----------------------------}
constructor TFancyLedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:=[csClickEvents, csCaptureMouse, csSetCaption];
  Enabled:=True;

  FEffect:=TLedEffect.Create;
  with FEffect do
  begin
    FaceColor:=clBtnFace;
    FrameColor:=clGray;
    FFrameWidth:=4;
    FHoverColor:=clBlue;
    FBtnShape:=btnRectangle;
    FShadowColor := clBtnShadow;
    FCornerRadius:=6;
    FTextStyle:=txLowered;
    FFrameStyle:=fmGradient;
    OnEffectChange:=UpdateChanges;
  end;

  FLed:=TDsLed.Create;
  with FLed do
  begin
    FLedColor := clRed;
    FLayot:=lyLeft;
//    FLedNormalColor := clRed;
    FLedHighlightColor := clLime;
    FLedBorderColor := clDkGray;
    FLedSize := 12;
    FLedBorderSize := 1;
    OnLedChange:=UpdateChanges;
    FLedEnabled:=true;
  end;

  FIsDown:=False;
  Height:=25;
  Visible:=True;
  Width:=97;
end;

destructor TFancyLedButton.Destroy;
begin
  inherited Destroy;
end;

function TFancyLedButton.SpotRGB(Color1, Color2 : TColor; Level: Byte): TColor;
var WarnaAwal, WarnaAkhir: TColor;
      AwalR, AwalG, AwalB,
      AkhirR, AkhirG, AkhirB: TColor;
      R, G, B: Byte;
begin
    WarnaAkhir:=ColorToRGB(Color1);
    WarnaAwal:=ColorToRGB(Color2);

    AwalR:=GetRValue(WarnaAwal); AkhirR:=GetRValue(WarnaAkhir);
    AwalG:=GetGValue(WarnaAwal); AkhirG:=GetGValue(WarnaAkhir);
    AwalB:=GetBValue(WarnaAwal); AkhirB:=GetBValue(WarnaAkhir);

    R:=Round((AwalR*Level+AkhirR*(100-Level))*0.01);
    G:=Round((AwalG*Level+AkhirG*(100-Level))*0.01);
    B:=Round((AwalB*Level+AkhirB*(100-Level))*0.01);
    Result:=RGB(R, G, B);
end;

procedure TFancyLedButton.Paint;
begin

  Canvas.Font:=Self.Font;
  Canvas.Brush.Style:=bsClear;

  if FEffect.FrameStyle=fmNone then DrawButtonArea
  else if FIsDown then DrawShapePressed
  else DrawShape;

  LayoutSetting;
  if FLed.FLedEnabled then DrawLed;
  WriteCaption;
end;

function TFancyLedButton.IsInRegion(X, Y: Integer): Boolean;
var Dia: Integer;
    MRgn: HRgn;
begin
  Result:=False;
  case FEffect.Shape of
    btnRectangle:
      begin
        Dia:=2*FEffect.CornerRadius;
        MRgn:=CreateRoundRectRgn(0, 0, Width, Height, Dia, Dia);
      end;
    btnCapsule:
      begin
        if Width<Height then Dia:=Width else Dia:=Height;
        MRgn:=CreateRoundRectRgn(0, 0, Width, Height, Dia, Dia);
      end;
    btnOval: MRgn:=CreateEllipticRgn(0, 0, Width, Height);
  end; //case

  try
    if PtInRegion(MRgn, X, Y) then Result:=True;
  finally
    DeleteObject(MRgn);
  end;
end;

procedure TFancyLedButton.DrawButtonArea;
var Dia: Integer;
begin
  if not (csDesigning in ComponentState) then Exit;
  Canvas.Pen.Style:=psDot;
  Canvas.Pen.Color:=clGray;
  case FEffect.Shape of
    btnRectangle:
      begin
        Dia:=2*FEffect.CornerRadius;
        Canvas.RoundRect(0, 0, Width, Height, Dia, Dia);
      end;
    btnCapsule:
      begin
        if Width<Height then Dia:=Width else Dia:=Height;
        Canvas.RoundRect(0, 0, Width, Height, Dia, Dia);
      end;
    btnOval: Canvas.Ellipse(0, 0, Width, Height);
  end; //case
end;

procedure TFancyLedButton.DrawShape;
var
  FC, Warna: TColor;
  R, G, B: Byte;
  AwalR, AwalG, AwalB,
  AkhirR, AkhirG, AkhirB,
  n, t, Dia: Integer;
  FRgn: HRgn;
  TempValue: real;
begin
  Warna:=ColorToRGB(FEffect.FaceColor);
  FC:=ColorToRGB(FEffect.FrameColor);

  AwalR:=GetRValue(FC); AkhirR:=GetRValue(Warna);
  AwalG:=GetGValue(FC); AkhirG:=GetGValue(Warna);
  AwalB:=GetBValue(FC); AkhirB:=GetBValue(Warna);

  t:=FEffect.FFrameWidth;
  for n:=0 to t-1 do
  begin
    TempValue:=Sqrt(t*t-Sqr(t-n))/t;
    R:=AwalR+Round(TempValue*(AkhirR-AwalR));
    G:=AwalG+Round(TempValue*(AkhirG-AwalG));
    B:=AwalB+Round(TempValue*(AkhirB-AwalB));
    Canvas.Brush.Color:=RGB(R, G, B);

    if FEffect.Shape=btnOval then
      FRgn:=CreateEllipticRgn(n, n, 1+Width-n, 1+Height-n)
    else
    begin
      if FEffect.Shape=btnCapsule then
        if Width<Height then Dia:=Width else Dia:=Height
      else Dia:=2*FEffect.CornerRadius;

      if (Dia-2*n)>0 then
        FRgn:=CreateRoundRectRgn(n, n, 1+Width-n, 1+Height-n, Dia-2*n, Dia-2*n)
      else FRgn:=CreateRectRgn(n, n, Width-n, Height-n);
    end;

    try
      FillRgn(Canvas.Handle, FRgn, Canvas.Brush.Handle);
    finally
      DeleteObject(FRgn);
    end;
  end;
end;

procedure TFancyLedButton.DrawShapePressed;
var Dia: Integer;
  FRgn: HRgn;

begin
  if FEffect.Shape=btnOval then
    FRgn:=CreateEllipticRgn(1, 1, Width-2, Height-2)
  else
  begin
    if FEffect.Shape=btnCapsule then
      if Width<Height then Dia:=Width else Dia:=Height
    else Dia:=2*FEffect.CornerRadius;

    if (Dia)>0 then
      FRgn:=CreateRoundRectRgn(1, 1, Width-2, Height-2, Dia-2, Dia-2)
    else FRgn:=CreateRectRgn(1, 1, Width-3, Height-3);
  end;

  try
    Canvas.Brush.Color:=clBlack;
    FillRgn(Canvas.Handle, FRgn, Canvas.Brush.Handle);
    OffsetRgn(FRgn, 1, 1);
    Canvas.Brush.Color:=SpotRGB(FEffect.FaceColor,FEffect.FrameColor,40); {--frame color: 40% face color: 60%--}
    FillRgn(Canvas.Handle, FRgn, Canvas.Brush.Handle);
    OffsetRgn(FRgn, 2, 2);
    Canvas.Brush.Color:=clBtnHighlight;
    FillRgn(Canvas.Handle, FRgn, Canvas.Brush.Handle);
    OffsetRgn(FRgn, -1, -1);
    Canvas.Brush.Color:=SpotRGB(FEffect.FaceColor,FEffect.FrameColor,15); {--frame color: 15% face color: 85%--}
    FillRgn(Canvas.Handle, FRgn, Canvas.Brush.Handle);
  finally
    DeleteObject(FRgn);
  end;
end;

procedure TFancyLedButton.WriteCaption;
var Flags: Word;
    R, TR: TRect;
    FTextColor: TColor;
begin
  R:=ClientRect; TR:=ClientRect;
  Canvas.Brush.Style:=bsClear;
  Flags:=DT_CENTER;
  Canvas.Font:=Font;

  if FInRegion and not FIsDown then FTextColor:=FEffect.HoverColor
  else FTextColor:=Self.Font.Color;

  with canvas do
  begin
    TR:=Rect(FTextL, FTextT, FTextR, Height);
    R:=TR;

    if (FEffect.TextStyle=txLowered) and not FIsDown then
    begin
      Font.Color:=clBtnHighLight;
      OffsetRect(TR, 1, 1);
      DrawText(Handle, PChar(Caption), Length(Caption), TR, Flags);
    end
    else if (FEffect.TextStyle=txLowered) and FIsDown then
    begin
      Font.Color:=clBtnHighLight;
      OffsetRect(TR, 2, 2);
      DrawText(Handle, PChar(Caption), Length(Caption), TR, Flags);
    end
    else if (FEffect.TextStyle=txRaised) and not FIsDown then
    begin
      Font.Color:=clBtnHighLight;
      OffsetRect(TR, -1, -1);
      DrawText(Handle, PChar(Caption), Length(Caption), TR, Flags);
    end
    else if (FEffect.TextStyle=txRaised) and FIsDown then
    begin
      Font.Color:=clBtnHighLight;
      DrawText(Handle, PChar(Caption), Length(Caption), TR, Flags);
    end
    else if (FEffect.TextStyle=txShadowed) and FIsDown then
    begin
      Font.Color:=FEffect.ShadowColor;
      OffsetRect(TR, 3, 3);
      DrawText(Handle, PChar(Caption), Length(Caption), TR, Flags);
    end
    else if (FEffect.TextStyle=txShadowed) and not FIsDown then
    begin
      Font.Color:=FEffect.ShadowColor;
      OffsetRect(TR, 2, 2);
      DrawText(Handle, PChar(Caption), Length(Caption), TR, Flags);
    end;

    if Enabled then Font.Color:=FTextColor
    else if (FEffect.TextStyle=txShadowed) and not Enabled then
      Font.Color:=clBtnFace
    else Font.Color:=FEffect.ShadowColor;
    if FIsDown then OffsetRect(R, 1, 1);
    DrawText(Handle, PChar(Caption), Length(Caption), R, Flags);
  end;
end;

procedure TFancyLedButton.LayoutSetting;
var n: Integer;
    wg, wt, ht: Integer;
    dist: Integer;
    TR: TRect;
    DC: HDC;
begin
  dist:=FLed.Distance;
  if FLed.Enabled then wg:=FLed.LedSize
    else wg:=0;
  FTextR := Width;

  TR:=ClientRect;
  DC := GetDC(0);
  Canvas.Handle := DC;
  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), TR, DT_CALCRECT or DT_EXPANDTABS);
  ht := TR.Bottom;
  wt := TR.Right;
  Canvas.Handle := 0;
  ReleaseDC(0, DC);

  case FLed.Layout of
    lyLeft  : begin
                if FAutoSize then
                  SetBounds (Left, Top, TR.Right+wg+dist+2+FEffect.FrameWidth*2, TR.Bottom+FEffect.FrameWidth*2);
                if Width>(wg+dist+wt) then FLedL:=(Width-wg-dist-wt) div 2
                 else FLedL:=2;
                if Height>wg then FLedT:=(Height-wg) div 2 else FLedT:=2;
                FTextL:=2+wg+dist;
                FTextT:=(Height-ht) div 2;
              end;
    lyRight : begin
                if FAutoSize then
                  SetBounds (Left, Top, TR.Right+wg+dist+2+FEffect.FrameWidth*2, TR.Bottom+FEffect.FrameWidth*2);
                if Width>(wg+dist+wt) then n:=(Width-wg-dist-wt) div 2
                else n:=2;
                FLedL:=Width-n-wg;
                if Height>wg then FLedT:=(Height-wg) div 2 else FLedT:=2;
                FTextL:= 0;
                FTextR:=Width-wg-dist;
                FTextT:=(Height-ht) div 2;
              end;
    lyTop   : begin
                if FAutoSize then
                  SetBounds (Left, Top, TR.Right+FEffect.FrameWidth*2, TR.Bottom+wg+dist+2+FEffect.FrameWidth);
                if Width>wg then FLedL:=(Width-wg) div 2 else FLedL:=2;
                if Height>(wg+dist+ht) then FLedT:=(Height-wg-dist-ht) div 2
                else FLedT:=0;
                FTextL:= 0;
                FTextT:=FLedT+wg+dist;
              end;
    lyBottom: begin
                if FAutoSize then
                  SetBounds (Left, Top, TR.Right+FEffect.FrameWidth*2, TR.Bottom+wg+dist+2+FEffect.FrameWidth);
                if Width>wg then FLedL:=(Width-wg) div 2 else FLedL:=2;
                if Height>(wg+dist+ht) then n:=(Height-wg-dist-ht) div 2
                else n:=2;
                FLedT:=Height-n-wg;
                FTextL:=0;
                FTextT:=(Height-wg-dist-ht) div 2;
              end;
  end;
end;

procedure TFancyLedButton.DrawLed;
var
    Radius, X, Y : integer;
begin
    if FIsDown then begin
        X := FLedL + 1;
        Y := FLedT + 1;
    end else begin
        X := FLedL;
        Y := FLedT;
    end;
    Radius := FLed.FLedSize;
    Canvas.Brush.Color := FLed.FLedBorderColor;
    Canvas.Ellipse (X, Y, X+Radius,Y+Radius);
    if Enabled then begin
      if FInRegion then
          Canvas.Brush.Color := FLed.FLedHighlightColor
        else Canvas.Brush.Color := FLed.FLedColor;
      if FIsDown then Canvas.Brush.Color :=
                 SpotRGB(FLed.FLedHighlightColor,FLed.FLedBorderColor,25);
    end
    else Canvas.Brush.Color := FEffect.FaceColor;
        Radius := Radius - FLed.FLedBorderSize;
        Canvas.Ellipse (X+FLed.FLedBorderSize, Y+FLed.FLedBorderSize,
                         X + Radius, Y + Radius);
end;

procedure TFancyLedButton.UpdateChanges(Sender: TObject);
begin
  if csLoading in ComponentState then Exit;
  Invalidate;
end;

procedure TFancyLedButton.MouseDown(Button: TMouseButton;
          Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if Button=mbRight then Exit;
  if not FInRegion and Enabled then begin
    FInRegion := True;
  end;

  FIsDown:=True;
  Invalidate;
end;

procedure TFancyLedButton.MouseUp(Button: TMouseButton;
          Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if Button=mbRight then Exit;
  if not FInRegion then Exit;
  inherited Click;
  FIsDown:=False;
  Invalidate;
end;

procedure TFancyLedButton.CMMouseEnter(var AMsg: TMessage);
begin
  inherited;
  if not FInRegion and Enabled then begin
    FInRegion := True;
    Invalidate;
  end;
end;

procedure TFancyLedButton.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  invalidate;
end;

procedure TFancyLedButton.CMTextChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TFancyLedButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled then
      begin inherited Click; Result:=1; end
    else inherited;
end;

procedure TFancyLedButton.CMMouseLeave(var AMsg: TMessage);
begin
  inherited;
  FInRegion:=False;
  FIsDown:=False;
  Invalidate;
end;

procedure TFancyLedButton.SetAutoSize(Value: Boolean);
begin
  if AutoSize <> Value then begin
    FAutoSize := Value;
    LayoutSetting;
  end;
end;

procedure TFancyLedButton.Click;
begin
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TFancyLedButton]);
  RegisterPropertyEditor(TypeInfo(TCaption), TFancyLedButton, 'Caption', THintProperty);
end;

end.

